function [v_projected] = projection_to_the_l1_sphere(v)
%Projects vector v to the l_1 sphere.
%----------------------
%Copyright (C) 2012 Zoltan Szabo (szzoli@cs.elte.hu)
%
%This file is part of the OSDL (Online Group-Structured Dictionary Learning) Matlab toolbox.
%
%OSDL is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with OSDL. If not, see <http://www.gnu.org/licenses/>.
%---------------------

z = 1;%=radius of the sphere

if sum(abs(v)) <= z
    v_projected = v;
else
    d = length(v); 
    absv = abs(v);
    v_sorted = sort(absv,'descend');
    v_sorted_cumsummed_v2 = (cumsum(v_sorted)-z) ./ [1:d].';
    I = v_sorted > v_sorted_cumsummed_v2;
    ro = find(I,1,'last');
    theta = v_sorted_cumsummed_v2(ro);
    v_projected = sign(v) .* max(absv-theta,0);
end
